//UTILS

// H3DModel2.cpp: implementation of the H3DModel class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "H3DModel2.h"
#include "3ds.h"
#include "HTextureStore.h"
#include "H3DModelBlendQueue.h"
#include "assert.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

t3DModel::t3DModel()
{
	m_numOfObjects   = 0;	
	m_numOfMaterials = 0;				
   m_numOfVerts     = 0;
	m_bHasDisplayList = false;
	m_loaded = false;
	m_retexturename[0] = '\0';
}

t3DModel::~t3DModel()
{
	int i;
	// zruseni textur
	for (i = 0; i < m_numOfMaterials; i++)
		htexture.RemoveTexture(m_pMaterials[i].textureId);
	
	// zruseni display listu modelu
	if (m_bHasDisplayList)
		glDeleteLists(m_displayList, 1);
	
	// zruseni display listu u objektu
	for(i = 0; i < m_numOfObjects; i++) 
		if (m_pObject[i].bHasDisplayList)
			glDeleteLists(m_pObject[i].displayList, 1);
		
	// zruseni vsech objektu v modelu
	for(i = 0; i < m_numOfObjects; i++)
	{
		// Free the faces, normals, vertices, and texture coordinates.
		delete [] m_pObject[i].pFaces;
		delete [] m_pObject[i].pNormals;
		delete [] m_pObject[i].pVerts;
		delete [] m_pObject[i].pTexVerts;
	}
}


// proste provedeme render celeho modelu
void t3DModel::Render(float x, float y, float z)
{
	if (!m_loaded) return;

#ifdef _HAVE_FRUSTUM_		
		if (!frustum.BoxInFrustum(m_boxMin.x + x, m_boxMin.y + y, m_boxMin.z + z,
										  m_boxMax.x + x, m_boxMax.y + y, m_boxMax.z + z
			                       )) 
		{
		  return;
		}	
#endif // _HAVE_FRUSTUM_
	
	
	glPushMatrix();

	//zkontrolujeme, zda uz nema hotovy display list
	if (m_bHasDisplayList) {
		glCallList(m_displayList);
		glPopMatrix();
		
#ifdef WANT_STATISTIC
		Tps_add(m_numOfVerts);
#endif
		
		return;
	}

   // Projdeme vsechny objekty a vsechny nakreslime
	for(int i = 0; i < m_numOfObjects; i++)
	{
		// Zkusime, zda je vubec objekt platny
		if(m_pObject.size() <= 0) break;

		// vezmeme si pointer na aktualne kresleny objekt
		t3DObject *pObject = &m_pObject[i];
		
		// zda je odjekt kreslitelny
		if (!pObject->wantDraw) continue;

// Zkusime, zda ho ma vubec smysl kreslit - zda je ve frustumu
#ifdef _HAVE_FRUSTUM_		
		if (!frustum.BoxInFrustum(pObject->boxMin.x + x, pObject->boxMin.y + y, pObject->boxMin.z + z,
										  pObject->boxMax.x + x, pObject->boxMax.y + y, pObject->boxMax.z + z
			                       )) 
		  continue;
#endif
		// podivame se, zda je nutne pouzit frontu na kresleni, jinak nakreslime sami
		if ( m_pMaterials[pObject->materialID].bNeedBlenQueue ) 
			hblendqueue.AddQueue(pObject, this);
		else
			RenderObject(pObject);
		
		
#ifdef WANT_STATISTIC
		Tps_add(pObject->numOfVerts);
#endif
		
	}
	
	// na konci osvetleni radsi vypnem
	glPopMatrix();
}

	// provadi render jednoho objektu - pouziva se u H3DModelBlendQueue
void t3DModel::RenderObject(const t3DObject *object)
{
	assert( object != NULL );
	
	// podivame, zda ma display list a kdyztak ho nakreslime ruco sami
	if (object->bHasDisplayList) 
		glCallList(object->displayList);
	else
		GiveObjectToOpenGL(object);
}



//////////////////////////////////////////////////////////////////////////////////////////

bool t3DModel::Load3DObject3DS(const char *filename)
{
	// zkusime nejprve soubor najit
	char filepath[MAX_PATH];
	strcpy(filepath, filename);
	if (!find_file(filepath)) return false;
	
	// importneme model
	CLoad3DS loader;
	loader.Import3DS(this, filepath);

	// Prihlidneme vsechny materialy a pripadne donacitame textury
	for(int i = 0; i < m_numOfMaterials; i++)
	{
		// Zkontrolujeme, jestli je u materialu textura, kterou by jsme mohli nacist
		if(strlen(m_pMaterials[i].strFile) > 0)
		{
			// nacteme texturu - pripadne retexturnem
			for (int c = 0; m_pMaterials[i].strFile[c] != '\0'; c++) // zmensime pismo
				if (m_pMaterials[i].strFile[c] >= 'A' && m_pMaterials[i].strFile[c] <= 'Z')  m_pMaterials[i].strFile[c] -= 'A' - 'a';
		
			// zkontrolujeme, zda budeme pretexturovavat
			if (strcmp(m_oldtexturename, m_pMaterials[i].strFile) == 0 || 
				(m_retexturename[0] != '\0' &&  m_oldtexturename[0] == '\0'))
				m_pMaterials[i].textureId = htexture.GetTextureAuto(m_retexturename);
			else
				m_pMaterials[i].textureId = htexture.GetTextureAuto(m_pMaterials[i].strFile);
			
			if ( htexture.m_texture[m_pMaterials[i].textureId]->alpha ) 
				m_pMaterials[i].bNeedBlenQueue = true;
			else
				m_pMaterials[i].bNeedBlenQueue = false;
				
		}
		else {
			m_pMaterials[i].textureId = -1;
			m_pMaterials[i].bNeedBlenQueue = false;
		}
	}
	
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////

bool t3DModel::LoadModel(const char* filename, int displayLists)
{
	if (m_loaded) return false; // muzeme nacist jen jeden model!!
	
	Load3DObject3DS(filename);

	// vypocitame normaly - aby objekty po nakresleni vypadali lepe
	ComputeNormals();
	// spocitame kazdemu objektu jeho "orezavaci krychli"
	ComputeObjectsCube();
	// spocitame display list
	ComputeDisplayLists(displayLists);
	
	m_loaded = true;
	return true;
}

// nacitani modelu se zamenou textury
bool t3DModel::LoadReTexturedModel(const char* filename, const char* texturename, const char* texturename_old, int displayLists)
{
	strcpy(m_retexturename, texturename);
	strcpy(m_oldtexturename, texturename_old);
	return LoadModel(filename, displayLists);
}

// pocitani normal v objektech - pro lepsi stinovani
void t3DModel::ComputeNormals()
{
	CVector3 vVector1, vVector2, vNormal, vPoly[3];

	// If there are no objects, we can skip this part
	if(m_numOfObjects <= 0)
		return;

	// What are vertex normals?  And how are they different from other normals?
	// Well, if you find the normal to a triangle, you are finding a "Face Normal".
	// If you give OpenGL a face normal for lighting, it will make your object look
	// really flat and not very round.  If we find the normal for each vertex, it makes
	// the smooth lighting look.  This also covers up blocky looking objects and they appear
	// to have more polygons than they do.    Basically, what you do is first
	// calculate the face normals, then you take the average of all the normals around each
	// vertex.  It's just averaging.  That way you get a better approximation for that vertex.

	// Go through each of the objects to calculate their normals
	for(int index = 0; index < m_numOfObjects; index++)
	{
		// Get the current object
		t3DObject *pObject = &(m_pObject[index]);

		// Here we allocate all the memory we need to calculate the normals
		CVector3 *pNormals		= new CVector3 [pObject->numOfFaces];
		CVector3 *pTempNormals	= new CVector3 [pObject->numOfFaces];

		pObject->pNormals		= new CVector3 [pObject->numOfVerts];

		// Go though all of the faces of this object
		int i;
		for( i=0; i < pObject->numOfFaces; i++)
		{
			// To cut down LARGE code, we extract the 3 points of this face
			vPoly[0] = pObject->pVerts[pObject->pFaces[i].vertIndex[0]];
			vPoly[1] = pObject->pVerts[pObject->pFaces[i].vertIndex[1]];
			vPoly[2] = pObject->pVerts[pObject->pFaces[i].vertIndex[2]];

			// Now let's calculate the face normals (Get 2 vectors and find the cross product of those 2)

			vVector1 = Vector(vPoly[0], vPoly[2]);		// Get the vector of the polygon (we just need 2 sides for the normal)
			vVector2 = Vector(vPoly[2], vPoly[1]);		// Get a second vector of the polygon

			vNormal  = Cross(vVector1, vVector2);		// Return the cross product of the 2 vectors (normalize vector, but not a unit vector)
			pTempNormals[i] = vNormal;					// Save the un-normalized normal for the vertex normals
			vNormal  = Normalize(vNormal);				// Normalize the cross product to give us the polygons normal

			pNormals[i] = vNormal;						// Assign the normal to the list of normals
		}

		//////////////// Now Get The Vertex Normals /////////////////

		CVector3 vSum = {0.0, 0.0, 0.0};
		CVector3 vZero = vSum;
		int shared=0;

		for (i = 0; i < pObject->numOfVerts; i++)			// Go through all of the vertices
		{
			for (int j = 0; j < pObject->numOfFaces; j++)	// Go through all of the triangles
			{												// Check if the vertex is shared by another face
				if (pObject->pFaces[j].vertIndex[0] == i ||
					pObject->pFaces[j].vertIndex[1] == i ||
					pObject->pFaces[j].vertIndex[2] == i)
				{
					vSum = AddVector(vSum, pTempNormals[j]);// Add the un-normalized normal of the shared face
					shared++;								// Increase the number of shared triangles
				}
			}

			// Get the normal by dividing the sum by the shared.  We negate the shared so it has the normals pointing out.
			pObject->pNormals[i] = DivideVectorByScaler(vSum, float(-shared));

			// Normalize the normal for the final vertex normal
			pObject->pNormals[i] = Normalize(pObject->pNormals[i]);

			vSum = vZero;									// Reset the sum
			shared = 0;										// Reset the shared
		}

		// Free our memory and start over on the next object
		delete [] pTempNormals;
		delete [] pNormals;
	}
}

// Spocitame stredy vsech objekty + velikost obklopujici krychle

void t3DModel::ComputeObjectsCube()
{
	CVector3 vVectorMin, vVectorMax, *pVector; // minimalni a maximalnihodnoty jednotlivych souradnic u  objektu
	
	CVector3 tVectorMin, tVectorMax;

	// Otestujeme, zda mame vubec k cemu pocitat obklopujici krychle
	if(m_numOfObjects <= 0)
		return;

	tVectorMin.x = 10000000;
	tVectorMin.y = 10000000;
	tVectorMin.z = 10000000;
	tVectorMax.x = -10000000;
	tVectorMax.y = -10000000;
	tVectorMax.z = -10000000;
	
	m_numOfVerts = 0;
	
	// Projdeme kazdy objekt a vypocitame mu okolni krychli
	for(int index = 0; index < m_numOfObjects; index++)
	{
		
		//  Pointer na aktualne zpracovavany objekt
		t3DObject *pObject = &(m_pObject[index]);

		// rovnou spocitame statisktiku
		m_numOfVerts += pObject->numOfVerts;
		
		// tohle je divne, ale treba bude objekt prazdny
		if (pObject->numOfVerts < 1) {
			pObject->cubeCenter.x = 0;
			pObject->cubeCenter.y = 0;
			pObject->cubeCenter.z = 0;
			pObject->cubeSide = 0;
		}
		
		// projdeme vsechny body, ktere objekt ma a hledame mezni souradnice
		vVectorMin = vVectorMax = pObject->pVerts[0];
		pVector = pObject->pVerts;
		for (int i = 0; i < pObject->numOfVerts; i++, pVector++)			
		{
			if (pVector->x < vVectorMin.x) vVectorMin.x = pVector->x;
			if (pVector->y < vVectorMin.y) vVectorMin.y = pVector->y;
			if (pVector->z < vVectorMin.z) vVectorMin.z = pVector->z;
				
			if (pVector->x > vVectorMax.x) vVectorMax.x = pVector->x;
			if (pVector->y > vVectorMax.y) vVectorMax.y = pVector->y;
			if (pVector->z > vVectorMax.z) vVectorMax.z = pVector->z;
		}
		
		// vypocitame stred objektu
		pObject->cubeCenter.x = (vVectorMin.x + vVectorMax.x)/2;
		pObject->cubeCenter.y = (vVectorMin.y + vVectorMax.y)/2;
		pObject->cubeCenter.z = (vVectorMin.z + vVectorMax.z)/2;

		// a ted jeste delku hrany - samozrejme nas zajima ta nejdelsi...
		pObject->cubeSide = vVectorMax.x - pObject->cubeCenter.x;
		
		if (pObject->cubeSide < vVectorMax.y - pObject->cubeCenter.y) 
			pObject->cubeSide = vVectorMax.y - pObject->cubeCenter.y;
		
		if (pObject->cubeSide < vVectorMax.z - pObject->cubeCenter.z) 
			pObject->cubeSide = vVectorMax.z - pObject->cubeCenter.z;
		
		// dopnime vektory do boxMin a boxMax
		pObject->boxMin = vVectorMin;
		pObject->boxMax = vVectorMax;
		
		// aktualizace u souradnic celeho modelu

		if (tVectorMin.x > vVectorMin.x) tVectorMin.x = vVectorMin.x;
		if (tVectorMin.y > vVectorMin.y) tVectorMin.y = vVectorMin.y;
		if (tVectorMin.z > vVectorMin.z) tVectorMin.z = vVectorMin.z;
				
		if (tVectorMax.x < vVectorMax.x) tVectorMax.x = vVectorMax.x;
		if (tVectorMax.y < vVectorMax.y) tVectorMax.y = vVectorMax.y;
		if (tVectorMax.z < vVectorMax.z) tVectorMax.z = vVectorMax.z;

//		jenom debug
//		fprintf(stdout, "New cube : %f, %f, %f, %f\n", pObject->cubeCenter.x, pObject->cubeCenter.y, pObject->cubeCenter.z, pObject->cubeSide);
	}
	
	m_modelCenter.x = (tVectorMin.x + tVectorMax.x)/2;
	m_modelCenter.y = (tVectorMin.y + tVectorMax.y)/2;
	m_modelCenter.z = (tVectorMin.z + tVectorMax.z)/2;
	
	// a ted jeste delku hrany - samozrejme nas zajima ta nejdelsi...
	m_cubeSide = tVectorMax.x - m_modelCenter.x;
		
	if (m_cubeSide < tVectorMax.y - m_modelCenter.y) 
		 m_cubeSide = tVectorMax.y - m_modelCenter.y;
	
	if (m_cubeSide < tVectorMax.z - m_modelCenter.z) 
		 m_cubeSide = tVectorMax.z - m_modelCenter.z;

	
	m_boxMin = tVectorMin;
	m_boxMax = tVectorMax;
}

void t3DModel::ComputeDisplayLists(int displayLists)
{
	GLuint dlists;
	int i;
	
	switch (displayLists) {
	
		// rekneme, ze nikdo nema zadne display listy
	case H3D_DL_NO:
		m_bHasDisplayList = false;
		for(i = 0; i < m_numOfObjects; i++)
			m_pObject[i].bHasDisplayList = false;
		break;
		
		// vytvorime display list pro kazdy objekt zvlast
	case H3D_DL_OBJECTS:
		m_bHasDisplayList = false;
		dlists = glGenLists(m_numOfObjects);
		for(i = 0; i < m_numOfObjects; i++) {
			m_pObject[i].displayList = dlists + i;
			if (m_pObject[i].wantDraw) {
				glNewList(m_pObject[i].displayList, GL_COMPILE);
				GiveObjectToOpenGL(&m_pObject[i]);
				glEndList();
				m_pObject[i].bHasDisplayList = true;
			} else {
				m_pObject[i].bHasDisplayList = false;
			}
		}
		break;
		
		// vytvorime display list pro cely model
	case H3D_DL_MODEL:
		m_bHasDisplayList = true;
		m_displayList = glGenLists(1);
		glNewList(m_displayList, GL_COMPILE);
		for(i = 0; i < m_numOfObjects; i++) {
			GiveObjectToOpenGL(&m_pObject[i]);
			m_pObject[i].bHasDisplayList = false;
		}
		glEndList();
		break;
	}
}

void t3DModel::GiveObjectToOpenGL(const t3DObject *pObject)
{
		if (!pObject->wantDraw) return;
	
		// Zjistime, zda ma objekt texturu a pripadne ji nastavime
		if (pObject->bHasTexture) 
			// Nastavime texturu podle materialu
			htexture.SetTexture(m_pMaterials[pObject->materialID].textureId);
		else htexture.SetNullTexture(); // Zakazeme texturovani
		
		// Kdyz objekt nema texturu, tak se podivame, zda ma alespon barevny material
		if(!pObject->bHasTexture && m_pMaterials.size() && pObject->materialID >= 0)
		{
			// Get and set the color that the object is, since it must not have a texture
			BYTE *pColor = m_pMaterials[pObject->materialID].color;
			
			// Assign the current color to this model
			glColor3ub(pColor[0], pColor[1], pColor[2]);
		}
		
		// zacneme kreslit trojuhelniky - 
		glBegin(GL_TRIANGLES);
			// Go through all of the faces (polygons) of the object and draw them
			for(int j = 0; j < pObject->numOfFaces; j++)
			{
				// Go through each corner of the triangle and draw it.
				for(int whichVertex = 0; whichVertex < 3; whichVertex++)
				{
					// Get the index for each point of the face
					int index = pObject->pFaces[j].vertIndex[whichVertex];

					// Give OpenGL the normal for this vertex.
					glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);

					// If the object has a texture associated with it, give it a texture coordinate.
					if(pObject->bHasTexture) {

						// Make sure there was a UVW map applied to the object or else it won't have tex coords.
						if(pObject->pTexVerts) {
							glTexCoord2f(pObject->pTexVerts[ index ].x, pObject->pTexVerts[ index ].y);
						}
					} 

					// Pass in the current vertex of the object (Corner of current face)
					glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
				}
			}

		glEnd();		// Konec kresleni trojuhelniku
}























































//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

H3DModel::H3DModel()
{
	m_loaded = false;
	m_retexturename[0] = '\0';
	m_oldtexturename[0] = '\0';
	m_loop = 0;
}

H3DModel::~H3DModel()
{
	delete [] m_model;
}

void H3DModel::Render(float x, float y, float z)
{
	if (!m_loaded) return;
	m_model[m_amodelID].Render(x, y, z);
}

void H3DModel::Animate(int milisec)
{
	// posuneme cas
	m_time += milisec;
	
	
	if (m_time < 0)
		if (m_loop) 
			while (m_time < 0) m_time += m_animtime;
		else
			m_time = 0;

	
	if (m_time >= m_animtime)
		if (m_loop) 
			m_time %= m_animtime;
		else
			m_time = m_animtime - 1;

		
	// podle casu urcime model
	m_amodelID = m_time / m_frametime;
}

void H3DModel::SetTime(int milisec)
{
	// posuneme cas
	m_time = milisec;
	Animate(0);
}

void H3DModel::AnimRestart() 
{
	m_amodelID = 0;
	m_time = 0;
}

  // nacte model - rozhodne. zda pouzit 3DS nebo HA3 (animace) nacitac
bool H3DModel::LoadModel(const char* filename, int displayLists, bool mirror_x, bool mirror_y, bool mirror_z)
{
	if (m_loaded) return false; // mozno nacist jen jednou model!!
	
	// test na jmeno souboru
	if (filename[0] == '\0') return false;

	// zkusime soubor najit
	char fn[255];
	strcpy(fn, filename);
	if (!find_file(fn)) return false;
	
	// vezmeme si pozadavek na D-listy
	m_displayLists = displayLists;
	if (mirror_x || mirror_y || mirror_z) displayLists = H3D_DL_NO;  // budou se vytvaret pozdeji - az v mirroru
	
	// rozhodneme, co nacist a hura na to
	char c = filename[strlen(filename)-1];
	if (c == '3') {
		if (!LoadHA3Model(fn, displayLists)) return false; // nacitame HA3
	}
	else {
		if (!Load3DSModel(fn, displayLists)) return false; // nacitame 3DS
	}
	
	// mirrornuti modelu, pripadne prepocteni a disply listy
	MirrorModel(mirror_x,mirror_y,mirror_z);
	 
	// kopie rozmeru i do hlavniho
	m_modelCenter = m_model->m_modelCenter;
	m_cubeSide	= m_model->m_cubeSide;
	m_boxMin	= m_model->m_boxMin;
	m_boxMax	= m_model->m_boxMax;
	m_boxCenter	= m_model->m_boxCenter;
	
	AnimRestart();
	m_loaded = true;
		
	return true;		
}



// nacitani modelu se zamenou textury
bool H3DModel::LoadReTexturedModel(const char* filename, const char* texturename, const char* texturename_old, int displayLists, bool mirror_x, bool mirror_y, bool mirror_z)
{
	strcpy(m_retexturename, texturename);
	strcpy(m_oldtexturename, texturename_old);
	return LoadModel(filename, displayLists, mirror_x, mirror_y, mirror_z);
}

// nacteni 3DS - na tom vlastne vubec nic neni
bool H3DModel::Load3DSModel(const char* filename, int displayLists)
{
	// default hodnoty pro animaci
	m_animtime = 1;
	m_models = 1;
	m_frametime = 1;
	
	m_model = new t3DModel [1];
	if (m_model == NULL) return false;
			
	if (!m_model->LoadReTexturedModel(filename, m_retexturename, m_oldtexturename, displayLists))
		return false;
	
	return true;
}

// nacita animacni model - predpoklada, ze uz filename je platna vec
bool H3DModel::LoadHA3Model(const char* filename, int displayLists)
{
	
// nacteni souboru
	FILE* f = fopen(filename, "rt");
	if (f == NULL) return false;
	
	fscanf(f, "%d\n", &m_animtime); // cas animace
	if (m_animtime <= 0) return false;
		
	fscanf(f, "%d\n", &m_models); // chteny pocet modelu
	if (m_models <= 0) return false;
	
	int mods = 0; // pocet modelu skutecnych
	fscanf(f, "%d\n", &mods); 
	if (mods <= 0) return false;
		
	char filepattern[255];  // jmeno souboru s modely
	fscanf(f, "%s\n", filepattern); 
	if (strlen(filepattern) <= 0) return false;

	// zda se ma provadet loop
	fscanf(f, "%d\n", &m_loop); 

	
	fclose(f);

// alokace pameti
	t3DModel **mod;
	int i;
	char fn[255];
	
	mod = new t3DModel* [mods+1];
	for (i = 0; i < mods; i++) {
		mod[i] = new t3DModel;
		sprintf(fn, filepattern, i+1); // aby se cislovalo od 1 - kuli grafikovi
		
		mod[i]->LoadReTexturedModel(fn, m_retexturename, m_oldtexturename, H3D_DL_NO);
	}
	
	if ( m_loop )
		mod[mods] = mod[0]; // aby se animace lepe "tocila" dokolecka - snaze dopocitavala
	else
		mod[mods] = mod[mods - 1]; // netocisi se animace
		
	
	// zatim nacteme bez dlistu - kuli alokaci v texturach, ale listy a normaly pak jeste jednou zmenime.
	m_model = new t3DModel [m_models];
	sprintf(fn, filepattern, 1);
	for (i = 0; i < m_models; i++) {
		m_model[i].LoadReTexturedModel(fn, m_retexturename, "", H3D_DL_NO);
	}		
	
// dopocitani animace
	int   dtime = m_animtime / mods; // casovy rozdil mezi skutecnymu modely
	float vtime = m_animtime / m_models; // casovy rozdil mezi tvorenymi modely 
	
	for (i = 0; i < m_models; i++) {
		int t  = (int)(vtime * i); // cas modelu
		int da = t / dtime; // drivejsi skutecny model (nejblizsi s nizsim casem
		t -= da*dtime; // rozdil casu od drivejsiho (casove) skutecneho modelu
		
		// provedeme preanimovani modelu
		CountAnim(mod[da], mod[da+1], &m_model[i], (float)t/(float)dtime);
		
		// vypocitame udaje k novemu modelu
 		m_model[i].ComputeNormals();     
		m_model[i].ComputeObjectsCube(); 
		m_model[i].ComputeDisplayLists(displayLists);
		
	}
	
	// prepocitame casy - zmenime rychlost animace :-( - ale bude to celociselne
	m_frametime = (int)vtime;
	m_animtime = m_frametime * m_models;
	
// uvolneni "zbytecne" pameti
	for (i = 0; i < mods; i++) delete mod[i];
	delete [] mod;
	

	return true;
}

void H3DModel::CountAnim(t3DModel* a, t3DModel* b, t3DModel *c, float p)
{
	float dp = 1 - p;
	
	// projdeme vsechny objekty
	for (int i = 0; i < a->m_numOfObjects; i++) {
		CVector3 *va = a->m_pObject[i].pVerts;
		CVector3 *vb = b->m_pObject[i].pVerts;
		CVector3 *vc = c->m_pObject[i].pVerts;
	
		//	v nich vsechny souradnice vrcholu
		for (int j = 0; j < a->m_pObject[i].numOfVerts; j++) {
			// a posuneme je
			vc->x =  (dp * va->x + p * vb->x);
			vc->y =  (dp * va->y + p * vb->y);
			vc->z =  (dp * va->z + p * vb->z);
			
			va++; vb++; vc++;
		}
		
	}
}

void H3DModel::MirrorModel(bool mirror_x, bool mirror_y, bool mirror_z)
{
	// alespon jeden musi byt true
	if (!mirror_x && !mirror_y && !mirror_z) return; 
	
	// zda bude potreba zmeni smer zapisu trojuhelniku
	int x = 0;
	if (mirror_x) x++;
	if (mirror_y) x++;
	if (mirror_z) x++;
	bool clockwize = false;
	if (x % 2 == 1) clockwize = true;
	
	t3DModel *a = m_model;
	for (int k = 0; k < m_models; k++, a++){
		// projedeme vsechny objekty
		for (int i = 0; i < a->m_numOfObjects; i++) {
			CVector3 *v = a->m_pObject[i].pVerts;
			//	v nich vsechny souradnice vrcholu
			for (int j = 0; j < a->m_pObject[i].numOfVerts; j++, v++) {
			// mirrornuti souradnic
				if (mirror_x) v->x = -v->x;
				if (mirror_y) v->y = -v->y;
				if (mirror_z) v->z = -v->z;
			}
			// a pripadne pretocime orientaci, aby model nebyl naruby
			if (clockwize) {
				tFace *f = a->m_pObject[i].pFaces;
				int t;
				for (int l = 0; l < a->m_pObject[i].numOfFaces; l++, f++) {
					// prohodime 3D souradnice 
					t = f->vertIndex[1];
					f->vertIndex[1] = f->vertIndex[2];
					f->vertIndex[2] = t;
					// a i pozice v texture
					t = f->coordIndex[1];
					f->coordIndex[1] = f->coordIndex[2];
					f->coordIndex[2] = t;
				}
			}
		}
		// nakonec musime model cely prepocitat
		a->ComputeNormals();
		a->ComputeObjectsCube();
		a->ComputeDisplayLists(m_displayLists);
	}
}
